#include "findcmd.h"
#include "sharedfolders.h"
#include "ftpserver.h"
#include <QTimer>

FINDcmd::FINDcmd(const QString &request, ServerThread *parent) : QObject(parent)
{
	this->parent()->send("150 Opening BINARY mode data connection for FIND " + request);

	connect(this->parent()->passv,SIGNAL(error(QAbstractSocket::SocketError)),this,SLOT(deleteLater()));
	connect(this->parent()->passv,SIGNAL(disconnected()),this,SLOT(deleteLater()));

	QMap<QString, SharedFolders::Folder> roots = SharedFolders::instance()->getFolderList();
	for(QMap<QString, SharedFolders::Folder>::iterator i = roots.begin() ; i != roots.end() ; ++i)
		workQueue.enqueue(QPair<QDir, QString>(QDir(i.value().getPath()), "/" + i.key() + "/"));

	dead = false;
	test = QRegExp(request, Qt::CaseInsensitive, QRegExp::Wildcard);
	k = 0;
	QTimer *timer = new QTimer(this);
	timer->setInterval(10);
	timer->setSingleShot(false);
	connect(timer,SIGNAL(timeout()),this,SLOT(process()));
	timer->start();
}

ServerThread *FINDcmd::parent() const
{
	return static_cast<ServerThread*>(QObject::parent());
}

void FINDcmd::finish()
{
	disconnect(this->parent()->passv,0,this,0);
	parent()->killDTP();
	parent()->send("226 Transfer complete");

	emit activity();
	emit finished();
	deleteLater();
	dead = true;
}

void FINDcmd::process()
{
	if (dead)
		return;
	if (workQueue.isEmpty() || parent()->mode != ServerThread::FIND)
	{
		finish();
		return;
	}

	QPair<QDir, QString> cur = workQueue.dequeue();

	QStringList entries = cur.first.entryList(QDir::AllDirs | QDir::NoSymLinks, QDir::NoSort);

	foreach(QString entry, entries)
	{
		if (entry == "." || entry == "..")
			continue;
		workQueue.enqueue(QPair<QDir, QString>(QDir(cur.first.absolutePath() + "/" + entry),
											   cur.second + entry + "/"));
	}

	entries = cur.first.entryList(QDir::AllEntries, QDir::NoSort);
	foreach(QString entry, entries)
	{
		if (entry == "." || entry == "..")
			continue;
		if (!test.exactMatch(entry))
			continue;

		QByteArray data = (cur.second + entry).toUtf8() + "\r\n";

		int written = 0;
		while (written != -1
			   && written < data.size()
			   && parent()->mode == ServerThread::FIND
			   && parent()->passv->isValid()
			   && parent()->passv->isOpen())
		{
			data.remove(0, written);
			written = parent()->passv->write(data);
			while (parent()->passv->isValid()
				&& parent()->mode == parent()->FIND
				&& parent()->passv->isOpen()
				&& parent()->passv->bytesToWrite() > 0)
			{
				parent()->passv->flush();
				parent()->passv->waitForBytesWritten(1);
			}
			FtpServer::pInstance->statsSend(data.size());
			if (k > 100)
			{
				emit activity();
				k = 0;
			}
			++k;
		}
		if (parent()->mode != ServerThread::FIND)
			break;
	}
}
