#include "ipcsocket.h"
#include "ipcconnection.h"
#include <QDebug>

IPCSocket *IPCSocket::pInstance = NULL;

IPCSocket *IPCSocket::instance(QObject *parent)
{
	if (pInstance)
		return pInstance;
	return new IPCSocket(parent);
}

IPCSocket::IPCSocket(QObject *parent) : QObject(parent)
{
	pInstance = this;
	sock = NULL;
	connect(this,SIGNAL(disconnecting()),this,SLOT(resetState()));
}

IPCSocket::IPCSocket(const QString &name, QObject *parent) : QObject(parent)
{
	pInstance = this;
	sock = NULL;
	connect(this,SIGNAL(disconnecting()),this,SLOT(resetState()));
	connectTo(name);
}

void IPCSocket::connectTo(const QString &name)
{
	emit disconnecting();
	sharedMemory.setKey(name);
	sharedMemory.attach();
	bool serverMode = sharedMemory.create(10);		// Server mode

	if (serverMode)
	{
		sock = new QLocalServer(this);
		connect(sock,SIGNAL(newConnection()),this,SLOT(processConnection()));
		connect(this,SIGNAL(disconnecting()),sock,SLOT(deleteLater()));
		if (!sock->listen(name))		// Try to listen (and become server)
			serverMode = false;
	}
	if (serverMode)
	{
		connect(sock,SIGNAL(destroyed()),this,SLOT(resetState()));
		currentState = Server;
		emit connected();
	}
	else
	{								// If it fails then a server is already listenning, let's connect to it
		emit disconnecting();
		currentState = NotConnected;
		QLocalSocket *s = new QLocalSocket(this);
		connect(s,SIGNAL(destroyed()),this,SLOT(resetState()));
		connect(this,SIGNAL(disconnecting()),s,SLOT(deleteLater()));
		IPCConnection *ipc = new IPCConnection(s);
		connect(ipc,SIGNAL(message(QByteArray)),this,SLOT(dispatchMessage(QByteArray)));
		connect(this,SIGNAL(sigSendMessage(QByteArray)),ipc,SLOT(sendMessage(QByteArray)));
		connect(s,SIGNAL(connected()),this,SLOT(relayConnectedSignal()));
		connect(s,SIGNAL(error(QLocalSocket::LocalSocketError)),this,SLOT(relayConnectedSignal()));
		s->connectToServer(name);
	}
}

void IPCSocket::processConnection()
{
	if (!sock)
		return;

	while(sock->hasPendingConnections())
	{
		QLocalSocket *s = sock->nextPendingConnection();
		IPCConnection *ipc = new IPCConnection(s);
		connect(ipc,SIGNAL(message(QByteArray)),this,SLOT(dispatchMessage(QByteArray)));
		connect(this,SIGNAL(sigSendMessage(QByteArray)),ipc,SLOT(sendMessage(QByteArray)));
	}
}

void IPCSocket::dispatchMessage(const QByteArray &msg)
{
	emit message(msg);
}

void IPCSocket::sendMessage(const QByteArray &msg)
{
	emit sigSendMessage(msg);
}

IPCSocket::State IPCSocket::state()
{
	return currentState;
}

void IPCSocket::resetState()
{
	currentState = NotConnected;
}

void IPCSocket::relayConnectedSignal()
{
	currentState = Client;
	emit connected();
}

IPCSocket::~IPCSocket()
{
	if (this == pInstance)
		pInstance = NULL;
}
