#include "mainwindow.h"
#include "ui_mainwindow.h"
#include "about.h"
#include "config.h"
#include "trayicon.h"
#include "stats.h"
#include "searchfile.h"
#include "ipcsocket.h"
#include <QSettings>

MainWindow::MainWindow(QWidget *parent)
    : QMainWindow(parent), ui(new Ui::MainWindow)
{
    ui->setupUi(this);

	new NetDiscovery;
	new Config(this);
	new TrayIcon(this);

	connect(IPCSocket::instance(this),SIGNAL(connected()),this,SLOT(checkIPCStatus()));
	connect(IPCSocket::instance(),SIGNAL(message(QByteArray)),this,SLOT(replyToIPCMessage(QByteArray)));
	IPCSocket::instance()->connectTo("qShare");

	connect(ui->action_AboutQt, SIGNAL(triggered()), this, SLOT(aboutQt()));
	connect(ui->action_About, SIGNAL(triggered()), this, SLOT(about()));
	connect(ui->action_Config, SIGNAL(triggered()), Config::instance(), SLOT(show()));
	connect(ui->action_Exit, SIGNAL(triggered()), this, SLOT(close()));
	connect(ui->action_Exit, SIGNAL(triggered()), qApp, SLOT(quit()));
	connect(NetDiscovery::pInstance, SIGNAL(serverListChanged()), this, SLOT(refreshServerList()));
	connect(ui->bRefresh, SIGNAL(clicked()), NetDiscovery::pInstance, SLOT(forceRefresh()));
	connect(ui->bOpen, SIGNAL(clicked()), this, SLOT(openServer()));
	connect(ui->serverList, SIGNAL(itemDoubleClicked(QListWidgetItem*)), this, SLOT(openServer()));
	connect(ui->actionStatistics, SIGNAL(triggered()), this, SLOT(showStatsDialog()));
	connect(ui->action_Find,SIGNAL(triggered()), this, SLOT(showSearchFileDialog()));

	QSettings settings;
	settings.beginGroup("mainwindow");
	resize(settings.value("size", size()).toSize());
	move(settings.value("pos", pos()).toPoint());
	settings.endGroup();

	refreshServerList();
}

MainWindow::~MainWindow()
{
    delete ui;
}

void MainWindow::showSearchFileDialog()
{
	SearchFile::instance()->show();
}

void MainWindow::showStatsDialog()
{
	Stats::instance()->show();
}

void MainWindow::about()
{
	About::instance()->show();
}

void MainWindow::aboutQt()
{
	QApplication::aboutQt();
}

void MainWindow::refreshServerList()
{
	lServer = NetDiscovery::pInstance->getServerList();
	QList<QListWidgetItem*> selection = ui->serverList->selectedItems();
	QString selectedName = selection.isEmpty() ? QString("") : selection.first()->text();

	ui->serverList->clear();
	foreach(Computer server, lServer)
	{
		QListWidgetItem *item;
		ui->serverList->addItem(item = new QListWidgetItem(QIcon(":/icons/computer"), server.getName()));
		if (server.getName() == selectedName)
			item->setSelected(true);
	}
}

void MainWindow::changeEvent(QEvent *e)
{
	QMainWindow::changeEvent(e);
	switch (e->type()) {
	case QEvent::LanguageChange:
		ui->retranslateUi(this);
		break;
	default:
		break;
	}
}

void MainWindow::closeEvent(QCloseEvent*)
{
	QSettings settings;
	settings.beginGroup("mainwindow");
	settings.setValue("size", size());
	settings.setValue("pos", pos());
	settings.endGroup();
}

void MainWindow::openServer()
{
	QList<QListWidgetItem*> items = ui->serverList->selectedItems();

	foreach(QListWidgetItem* item, items)
	{
		foreach(Computer c, lServer)
		{
			if (c.getName() == item->text())
				c.open();
		}
	}
}

void MainWindow::checkIPCStatus()
{
	if (IPCSocket::instance()->state() == IPCSocket::Client)		// qShare is already started
	{
		// Parse command line arguments and send them to the running instance
		QStringList args = QCoreApplication::arguments();
		// Show main window by default (if no arguments is given)
		if (args.size() == 1)
			IPCSocket::instance()->sendMessage("MainWindow::show()");
		foreach(QString arg, args)
			IPCSocket::instance()->sendMessage(arg.toAscii());
		exit();
	}

	if (IPCSocket::instance()->state() == IPCSocket::Server)
	{
		// Parse command line arguments and send them to replyToIPCMessage
		QStringList args = QCoreApplication::arguments();
		foreach(QString arg, args)
			replyToIPCMessage(arg.toAscii());
	}
}

void MainWindow::replyToIPCMessage(const QByteArray &msg)
{
	if (msg == "MainWindow::show()")
		show();
	else if (msg == "About::show()")
		About::instance()->show();
	else if (msg == "Config::show()")
		Config::instance()->show();
	else if (msg == "Stats::show()")
		Stats::instance()->show();
	else if (msg == "SearchFile::show()")
		SearchFile::instance()->show();
	else if (msg == "MainWindow::hide()")
		hide();
	else if (msg == "About::hide()")
		About::instance()->hide();
	else if (msg == "Config::hide()")
		Config::instance()->hide();
	else if (msg == "Stats::hide()")
		Stats::instance()->hide();
	else if (msg == "SearchFile::hide()")
		SearchFile::instance()->hide();
	else if (msg == "MainWindow::exit()" || msg == "exit")
		exit();
}

void MainWindow::exit()
{
	QTimer *timer = new QTimer;
	timer->setInterval(1);
	timer->setSingleShot(false);
	connect(timer,SIGNAL(timeout()),qApp,SLOT(quit()));
	timer->start();
}
