#include "stats.h"
#include "ui_stats.h"
#include "ftpserver.h"
#include <QTimer>
#include <QGraphicsScene>
#include <QRgb>

Stats *Stats::pInstance = NULL;

Stats *Stats::instance()
{
	if (pInstance)
		return pInstance;
	return new Stats;
}

void Stats::deleteInstance()
{
	if (pInstance)
		pInstance->deleteLater();
}

Stats::Stats(QWidget *parent) :
    QDialog(parent),
    m_ui(new Ui::Stats)
{
	pInstance = this;

    m_ui->setupUi(this);

	m_ui->chart->setScene(new QGraphicsScene);

	prevSent = 0;
	prevRecv = 0;

	QTimer *timer = new QTimer;
	connect(timer,SIGNAL(timeout()),this,SLOT(updateChart()));
	connect(timer,SIGNAL(timeout()),m_ui->stats,SLOT(repaint()));
	timer->setInterval(250);
	timer->setSingleShot(false);
	timer->start();
	clock.start();
}

Stats::~Stats()
{
    delete m_ui;

	pInstance = NULL;
}

void Stats::changeEvent(QEvent *e)
{
    QDialog::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
        m_ui->retranslateUi(this);
        break;
    default:
        break;
    }
}

void Stats::showEvent(QShowEvent *)
{
	updateStats();
}

void Stats::updateStats()
{
	if (!isVisible())
		return;

	if (!FtpServer::serverRunning())
	{
		m_ui->users->setRowCount(0);
		return;
	}

	setUpdatesEnabled(false);
	QList<FtpServer::ConnectionInfo> lInfo = FtpServer::getClientsInfo();
	m_ui->users->setRowCount(lInfo.size());
	int i(0);
	foreach(FtpServer::ConnectionInfo info, lInfo)
	{
		m_ui->users->setItem(i, 0, new QTableWidgetItem(info.user));
		m_ui->users->setItem(i, 1, new QTableWidgetItem(info.address));
		m_ui->users->setItem(i, 2, new QTableWidgetItem(info.activity));
		++i;
	}

	QStringList sInfo = FtpServer::getServerInfo();
	for(int i = 0 ; i < sInfo.size() ; ++i)			// Make sure there are items there
		if (!m_ui->stats->item(i,0))
			m_ui->stats->setItem(i, 0, new QTableWidgetItem());

	quint64 size = sInfo[0].toULongLong();			// Sent bytes
	QString bytesStr;
	if (!(size >> 10))
		bytesStr = QString::number(size) + ' ' + tr("Bytes");
	else if (!(size >> 20))
		bytesStr = QString::number(size / 1024.0, 'f', 3) + ' ' + tr("KB");
	else if (!(size >> 30))
		bytesStr = QString::number((size >> 10) / 1024.0, 'f', 3) + ' ' + tr("MB");
	else if (!(size >> 40))
		bytesStr = QString::number((size >> 20) / 1024.0, 'f', 3) + ' ' + tr("GB");
	else
		bytesStr = QString::number((size >> 30) / 1024.0, 'f', 3) + ' ' + tr("TB");
	m_ui->stats->item(0,0)->setText(bytesStr);

	size = sInfo[1].toULongLong();				// Received bytes
	bytesStr.clear();
	if (!(size >> 10))
		bytesStr = QString::number(size) + ' ' + tr("Bytes");
	else if (!(size >> 20))
		bytesStr = QString::number(size / 1024.0, 'f', 3) + ' ' + tr("KB");
	else if (!(size >> 30))
		bytesStr = QString::number((size >> 10) / 1024.0, 'f', 3) + ' ' + tr("MB");
	else if (!(size >> 40))
		bytesStr = QString::number((size >> 20) / 1024.0, 'f', 3) + ' ' + tr("GB");
	else
		bytesStr = QString::number((size >> 30) / 1024.0, 'f', 3) + ' ' + tr("TB");
	m_ui->stats->item(1,0)->setText(bytesStr);
	for(int i = 2 ; i < sInfo.size() ; ++i)
		m_ui->stats->item(i, 0)->setText(sInfo[i]);

	setUpdatesEnabled(true);
}

void Stats::updateChart()
{
	QStringList sInfo = FtpServer::getServerInfo();

	quint64 elapsed = clock.restart();
	quint64 timeCoef = (quint64(250) << 32) / elapsed;
	quint64 n = elapsed / 250;

	for(quint64 i = 0 ; i < n ; ++i)
	{
		connections.push_back( sInfo[2].toInt() * timeCoef >> 32 );
		bytesOut.push_back((sInfo[0].toLongLong() - prevSent) * timeCoef >> 32);
		bytesIn.push_back((sInfo[1].toLongLong() - prevRecv) * timeCoef >> 32);
	}
	prevSent = sInfo[0].toULongLong();
	prevRecv = sInfo[1].toULongLong();
	while (bytesIn.size() > m_ui->chart->width() - 8)
		bytesIn.pop_front();
	while (bytesOut.size() > m_ui->chart->width() - 8)
		bytesOut.pop_front();
	while (connections.size() > m_ui->chart->width() - 8)
		connections.pop_front();

	while (bytesIn.size() < m_ui->chart->width() - 8)
		bytesIn.push_front(0);
	while (bytesOut.size() < m_ui->chart->width() - 8)
		bytesOut.push_front(0);
	while (connections.size() < m_ui->chart->width() - 8)
		connections.push_front(0);

	if (!isVisible())
		return;

	setUpdatesEnabled(false);
	m_ui->chart->scene()->clear();
	QPen pen(QColor(255,0,0));
	qint64 max = 1;
	for(int i = 0 ; i < bytesOut.size() ; ++i)
		max = qMax(max, bytesOut[i]);
	double coef = 0.667 * m_ui->chart->height() / double(max);
	for(int i = 1 ; i < bytesOut.size() ; ++i)
		m_ui->chart->scene()->addLine(i - 1, -bytesOut[i-1] * coef, i, -bytesOut[i] * coef, pen);

	max = 1;
	for(int i = 0 ; i < bytesIn.size() ; ++i)
		max = qMax(max, bytesIn[i]);
	coef = 0.667 * m_ui->chart->height() / double(max);
	pen.setColor(QColor(0,0,255));
	for(int i = 1 ; i < bytesIn.size() ; ++i)
		m_ui->chart->scene()->addLine(i - 1, -bytesIn[i-1] * coef, i, -bytesIn[i] * coef, pen);

	max = 1;
	for(int i = 0 ; i < connections.size() ; ++i)
		max = qMax(max, connections[i]);
	coef = 0.667 * m_ui->chart->height() / double(max);
	pen.setColor(QColor(0,255,0));
	for(int i = 1 ; i < connections.size() ; ++i)
		m_ui->chart->scene()->addLine(i - 1, -connections[i-1] * coef, i, -connections[i] * coef, pen);
	setUpdatesEnabled(true);
}
