#include "storcmd.h"
#include "ftpserver.h"

STORcmd::STORcmd(const QString &filename, bool append, ServerThread *parent) : QObject(parent), file(filename)
{
	dead = false;
	if (!append)
		file.open(QFile::WriteOnly);
	else
		file.open(QFile::WriteOnly | QFile::Append);
	k = 0;
	if (file.exists() && file.isWritable() && file.isOpen())
	{
		connect(this->parent()->passv,SIGNAL(readyRead()),this,SLOT(readData()));
		connect(this->parent()->passv,SIGNAL(error(QAbstractSocket::SocketError)),this,SLOT(deleteLater()));
		connect(this->parent()->passv,SIGNAL(disconnected()),this,SLOT(disconnected()));

		this->parent()->send("150 File status okay; about to open data connection.");
	}
	else
	{
		this->parent()->send("451 Requested action aborted. Local error in processing.");
		this->parent()->killDTP();
		dead = true;
		emit activity();
		deleteLater();
	}
}

STORcmd::~STORcmd()
{
}

void STORcmd::disconnected()
{
	disconnect(this->parent()->passv,0,this,0);
	file.flush();
	file.close();

	parent()->killDTP();

	parent()->send("250 Transfer complete");

	emit activity();
	emit finished();

	deleteLater();

	dead = true;
}

ServerThread *STORcmd::parent() const
{
	return static_cast<ServerThread*>(QObject::parent());
}

void STORcmd::readData()
{
	if (dead)
		return;
	QByteArray data = parent()->passv->readAll();

	FtpServer::pInstance->statsRecv(data.size());
	if (k > 100)
	{
		emit activity();
		k = 0;
	}
	++k;
	file.write(data);

	if (parent()->mode != ServerThread::STOR && parent()->mode != ServerThread::APPE)
	{
		this->parent()->send("451 Requested action aborted. Local error in processing.");
		this->parent()->killDTP();
		deleteLater();
		dead = true;
	}
}
